#!/usr/bin/env ruby
#
# PDFlib PLOP DS: 
# generate a dynamic visualization document in memory with PDFlib,
# and digitally sign a second document, using the dynamic document
# for signature visualization.
#

require './PDFlib'
require './PLOP'
           
# This is where input files live. Adjust as necessary. 
searchpath = "../data"
    
# Basic signature option

sign_opts =
    "ltv=try " +        # use ltv=full to ensure LTV status 
    #when using PKCS11
    # "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "

if (ARGV.length != 2 )
    raise "usage: dynamicsign <infile> <outfile>\n"
end


begin
    # ------------- Step 1: generate the document in memory with PDFlib ------------
    p = PDFlib.new

    # open new PDF document  in memory

    p.begin_document("", "")

    p.set_info("Creator", "dynamicsign.rb")
    p.set_info("Author", "Rainer Ploeckl")
    p.set_info("Title", "Dynamic signature visualization document")


    p.begin_page_ext(300, 50, "")
    now = Time.now
    timebuf = "Digitally signed " + now.strftime("%b %d, %Y") + " at " + now.strftime("%H:%M:%S")

    textoptlist = "fontsize=10 fontname=Helvetica encoding=unicode textformat=utf8 fillcolor=red"
    p.fit_textline(timebuf, 10, 10, textoptlist)
    p.end_page_ext("")

    p.end_document("")

    # fetch the memory buffer containing the PDF document
    contents = p.get_buffer()

rescue PDFlibException => pe
    print pe.backtrace.join("\n")
    printf("PDFlib exception occurred in dynamicsign:\n")
    print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                        ": " + pe.get_errmsg + "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    p.delete() if p
end

# ------------------ Step 2: sign the document -----------------
    

begin
    # create a new plop object
    plop = PLOP.new

    plop.set_option("searchpath={{" + searchpath + "}}")

    # Create a virtual PVF file from the data in memory
    plop.create_pvf("/pvf/visdoc.pdf", contents, "")
    if ((doc = plop.open_document(ARGV[0], "")) == -1)
      raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end
    # open visualization  document from PVF file
    if ((visdoc = plop.open_document("/pvf/visdoc.pdf", "")) == -1)
        raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end
    
    # Add field option for visualization to base signature options
    optlist = sign_opts + " field={visdoc=" + visdoc.to_s + " rect={10 10 adapt adapt}}"
    if (plop.prepare_signature(optlist) == -1)
        raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end

    # sign the output file
    if (plop.create_document(ARGV[1], "input=#{doc}") == -1)
        raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end
        
    # close the input document
    plop.close_document(doc, "")

    # close the visualization document
    plop.close_document(visdoc, "")

    # delete PVF file containing input document
    if (plop.delete_pvf("/pvf/visdoc.pdf") == -1)
        raise "ERROR: " + plop.get_errmsg()
    end

rescue PLOPException => pe
    print pe.backtrace.join("\n")
    printf("PLOP exception occurred in dynamicsign:\n")
    print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                        ": " + pe.get_errmsg + "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    plop.delete() if plop
end
